class command {    // базовый класс command с перегруженной операцией operator()
public:	
   virtual void operator() ( Y86*, struct comnd* ) = 0;
};

class noop : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
      if( "halt" == c->mnemo ) 
         p->state = p->HALT;
      else
         p->IP += c->length;
   }
};

class rrmovl : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
	  uint8_t op = p->ram[ p->IP + 1 ];
	  uint8_t rB = op & 0xf, rA = ( op >> 4 ) & 0xf;
	  p->R[ rB ] = p->R[ rA ];  
      p->IP += c->length;
   }
};

class irmovl : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
	  uint8_t rB = p->ram[ p->IP + 1 ] & 0xf;
	  uint32_t op = p->value_at( p->IP + 2 );
      p->R[	rB ] = op;   
      p->IP += c->length;
   }
};

class rmmovl : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
	  uint8_t op = p->ram[ p->IP + 1 ];
   	  uint8_t rB = op & 0xf, rA = ( op >> 4 ) & 0xf;
	  uint32_t sh = p->value_at( p->IP + 2 ),
	           addr = p->R[ rB ] + sh;
      p->save32reg( rA, addr );	            
      p->IP += c->length;
   }
};

class mrmovl : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
	  uint8_t op = p->ram[ p->IP + 1 ];
   	  uint8_t rB = op & 0xf, rA = ( op >> 4 ) & 0xf;
	  uint32_t sh = p->value_at( p->IP + 2 ),
	           addr = p->R[ rB ] + sh;
      p->R[ rA ] = p->value_at( addr );
      p->IP += c->length;
   }
};

class arithm : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
	  uint8_t op = p->ram[ p->IP + 1 ];
   	  uint8_t rA = ( op >> 4 ) & 0xf, rB = op & 0xf; // [rA] op [rB] => [rB]
      uint32_t v1 = p->R[ rA ], v2 = p->R[ rB ];
      int64_t res = v2;
      bool ok = true; 
      if( "addl" == c->mnemo )      res += (int64_t)v1;
      else if( "subl" == c->mnemo ) res -= (int64_t)v1;
      else if( "andl" == c->mnemo ) res = v2 & v1;
      else if( "xorl" == c->mnemo ) res = v2 ^ v1;
      else {
		 ok = false;
         cerr << "недопустимый код арифметической операции" << endl;   
	  }
	  if( ok ) {
	     p->R[ rB ] = (uint32_t)( res & 0xffffffffL );
         p->ZF = 0 == p->R[ rB ];
         if( "addl" == c->mnemo || "subl" == c->mnemo ) {
            p->SF = p->R[ rB ] < 0;
            p->OF = p->SF ? ( res & 0x100000000L ) == 0L :
                            ( res & 0x100000000L ) != 0L;
         }
         else 
			p->SF = p->OF = false; 
      }
      p->IP += c->length;
   }
};

class jmp : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
	  if( ( "jmp" == c->mnemo ) ||
	      ( "jle" == c->mnemo && ( p->ZF || p-> SF ) ) ||	  	  
          ( "jl"  == c->mnemo && ( p->SF ) ) ||
          ( "je"  == c->mnemo && ( p->ZF ) ) ||
          ( "jne" == c->mnemo && ( !p->ZF ) ) ||
          ( "jge" == c->mnemo && ( p->ZF || !p-> SF ) ) ||
          ( "jg"  == c->mnemo && ( !p-> SF ) ) ) {
	     uint32_t addr = p->value_at( p->IP + 1 );  
	     p->IP = addr;
	  }
      else 
         p->IP += c->length;
   }
};

const int SP = 4; // номер регистра указателя стека %esp

class call : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
      p->R[ SP ] -= sizeof( uint32_t );
      p->save32( p->IP + c->length, p->R[ SP ] );
      uint32_t addr = p->value_at( p->IP + 1 ); 
      p->IP = addr; 
   }
};

class ret : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
      uint32_t addr = p->value_at( p->R[ SP ] );
      p->R[ SP ] += sizeof( uint32_t ); 
      p->IP = addr; 
   }
};

class push_pop : public command {
   virtual void operator() ( Y86* p, struct comnd* c ) {
	  uint8_t rA = p->ram[ p->IP + 1 ] >> 4 & 0xf; // номер регистра 
	  if( "pushl" == c->mnemo ) {
         p->R[ SP ] -= sizeof( uint32_t );
         p->save32reg( rA, p->R[ SP ] );
	  }
	  else {
		 p->load32reg( rA, p->R[ SP ] ); 
         p->R[ SP ] += sizeof( uint32_t ); 
	  }
      p->IP += c->length;
   }
};

static bool init_table( void ) {    // функция стартовой инициализации
   map< string, command* > op_table = {
      { "nop"   , new noop     },   // nop
      { "halt"  , new noop     },   // halt 
      { "rrmovl", new rrmovl   },   // rrmovl    rA -> rB
      { "irmovl", new irmovl   },   // irmovl     V -> rB
      { "rmmovl", new rmmovl   },   // rmmovl    rA -> D(rB)  
      { "mrmovl", new mrmovl   },   // mrmovl D(rB) -> rA 
      { "addl"  , new arithm   },   // addl      rA + rB -> rB 
      { "subl"  , new arithm   },   // subl      rA - rB -> rB
      { "andl"  , new arithm   },   // andl      rA & rB -> rB 
      { "xorl"  , new arithm   },   // xorl      rA ^ rB -> rB
      { "jmp"   , new jmp      },   // jmp       A
      { "jle"   , new jmp      },   // jle       A
      { "jl"    , new jmp      },   // jl        A  
      { "je"    , new jmp      },   // je        A 
      { "jne"   , new jmp      },   // jne       A
      { "jge"   , new jmp      },   // jge       A
      { "jg"    , new jmp      },   // jg"       A
      { "call"  , new call     },   // call      A
      { "ret"   , new ret      },   // ret
      { "pushl" , new push_pop },   // pushl     rA
      { "popl"  , new push_pop }    // popl      rA
   };
   for( int i = 0; i < ncomnd; i++ ) 
      try {
         comnds[ i ].func = (const void*)op_table.at( comnds[ i ].mnemo );
      }
      catch( out_of_range const& e ) {
         cerr << "несоответствие таблиц: " << comnds[ i ].mnemo << endl; 
		 return false;
      }  
   if( ncomnd != op_table.size() ) {
      cerr << "несоответствие размеров таблиц" << endl; 
	  return false;
   }
   return true;
}
