//-------------------------- разбор операндов -------------------------------
typedef string (*op_func)( smatch&, map< string, uint32_t >& );

const map< string, char > registers = {
   { "ax", '0' }, { "cx", '1' }, { "dx", '2' }, { "bx", '3' },
   { "sp", '4' }, { "bp", '5' }, { "si", '6' }, { "di", '7' } };

string oper0( smatch& m, map< string, uint32_t >& lbl ) {
   return "";
} 

string irmovl( smatch& m, map< string, uint32_t >& lbl ) {
   char op[ 3 ] = "f0";
   op[ 1 ] = registers.at( m[ 3 ].str() );
   string ret( op );
   if( '$' == m[ 2 ].str()[ 0 ] )                      // числовой операнд
      ret += load_arg( get_number( m[ 2 ].str().substr( 1 ) ) );      
   else 
      try {                                            // адрес метки
         ret += load_arg( lbl.at( m[ 2 ].str() ) );
      }
      catch( out_of_range const& e ) {
         cout << "неопределённое имя метки: " << m[ 2 ].str() << endl;
         exit( 5 );
      }
   return ret;
}

string rmmovl( smatch& m, map< string, uint32_t >& lbl ) {
   char op[ 3 ] = "00";
   op[ 0 ] = registers.at( m[ 2 ].str() );
   op[ 1 ] = registers.at( m[ 4 ].str() );
   return string( op ) + load_arg( get_number( m[ 3 ].str() ) );
}

string mrmovl( smatch& m, map< string, uint32_t >& lbl ) {
   char op[ 3 ] = "00";
   op[ 0 ] = registers.at( m[ 4 ].str() );
   op[ 1 ] = registers.at( m[ 3 ].str() );
   return string( op ) + load_arg( get_number( m[ 2 ].str() ) );
}

string arithm( smatch& m, map< string, uint32_t >& lbl ) {
   char op[ 3 ] = "00";
   op[ 0 ] = registers.at( m[ 2 ].str() );
   op[ 1 ] = registers.at( m[ 3 ].str() );
   return string( op );
}

string jmp( smatch& m, map< string, uint32_t >& lbl ) {
   uint32_t addr;
   try {
      addr = lbl.at( m[ 2 ].str() );
   }
   catch( exception const& e ) {
      cout << "неопределённое имя метки: " << m[ 2 ].str() << endl;
      exit( 5 );
   }
   return load_arg( addr );
}

string push_pop( smatch& m, map< string, uint32_t >& lbl ) {
   char op[ 3 ] = "0f";
   op[ 0 ] = registers.at( m[ 2 ].str() );
   return string( op );
}

static int init = []( void ) -> bool {                 // анонимная функция стартовой инициализации
   const map< string, op_func > op_table = {
      { "nop"   , oper0    },        // nop
      { "halt"  , oper0    },        // halt 
      { "rrmovl", arithm   },        // rrmovl    rA, rB
      { "irmovl", irmovl   },        // irmovl     V, rB
      { "rmmovl", rmmovl   },        // rmmovl    rA, D(rB)  
      { "mrmovl", mrmovl   },        // mrmovl D(rB), rA 
      { "addl"  , arithm   },        // addl      rA, rB 
      { "subl"  , arithm   },        // subl      rA, rB
      { "andl"  , arithm   },        // andl      rA, rB 
      { "xorl"  , arithm   },        // xorl      rA, rB
      { "jmp"   , jmp      },        // jmp       A
      { "jle"   , jmp      },        // jle       A
      { "jl"    , jmp      },        // jl        A  
      { "je"    , jmp      },        // je        A 
      { "jne"   , jmp      },        // jne       A
      { "jge"   , jmp      },        // jge       A
      { "jg"    , jmp      },        // jg"       A  
      { "call"  , jmp      },        // call      A 
      { "ret"   , oper0    },        // ret
      { "pushl" , push_pop },        // pushl     rA
      { "popl"  , push_pop }         // popl      rA 
   };
   for( int i = 0; i < ncomnd; i++ ) 
      try {
         comnds[ i ].func = (const void*)op_table.at( comnds[ i ].mnemo );
      }
      catch( out_of_range const& e ) {
         cerr << "несоответствие таблиц: " << comnds[ i ].mnemo << endl; 
		 exit( 6 );
      }  
   if( ncomnd != op_table.size() ) {
      cerr << "несоответствие таблиц" << endl; 
	  exit( 6 );	   
   }
   return ncomnd; 
} (); 
