#ifndef PROCESSOR_H
#define PROCESSOR_H
#include "common.h"

class Y86 : private map< uint32_t, string >  {
   string line;                                             // последняя прочитанная строка (ошибочная!)
protected:
   bool OK;
   unsigned count;                                          // счётчик выполненных команд
   void load( uint32_t addr, const vector<int>& ops );      // загрузка команды в mem
public:
   enum { LOAD, RUN, HALT } state;
   uint32_t IP;
   bool ZF, SF, OF;                                         // флаги операций 
   const static uint8_t rnum = 8;
   uint32_t R[ rnum ];                                      // регистры процессора
   vector<uint8_t> ram;                                     // массив байтовой памяти
   Y86( void ) : OK( false ), state( HALT ) {}
   void save32( uint32_t val, uint32_t addr );              // сохранить 32-значение в память  
   inline void save32reg( uint8_t nreg, uint32_t addr ) {   // сохрапить регистр в память  
      save32( R[ nreg ], addr );
   }
   uint32_t value_at( uint32_t addr );                      // считать значение из памяти
   inline void load32reg( uint8_t nreg, uint32_t addr ) {   // загрузить регистр из памяти
      R[ nreg ] = value_at( addr );
   }
   static const bool init;                                  // успех начальной инициализации 
   inline bool operator !( void ) { return !OK; }           // корректный загруженный код
   inline string& errline( void ) { return line; }
   string current_line( void ); 
   string get_state( void );
   string get_regs( void );
   string dump( void );                                     // дамп памяти
   string compare( Y86& );
   bool step( void );
   inline void execute( int n = 10 ) { 
      while( ( state != HALT ) && ( n-- != 0 ) && step() );
   }
   friend istream& operator >>( istream& inp, Y86& pr );    // загрузчик кода
};

#endif
